/*
 * See the file "LICENSE" for the full license governing this code.
 */
package up;

/**
 * A binary rule of a PCFG. This is a rule of the form A → B C.
 * 
 * @author Marco Kuhlmann <marco.kuhlmann@lingfil.uu.se>
 */
public class BinaryRule extends Rule {

	/**
	 * The left-hand side of this rule.
	 */
	private final int lhs;
	/**
	 * The first category on the right-hand side of this rule.
	 */
	private final int rhs1;
	/**
	 * The second category on the right-hand side of this rule.
	 */
	private final int rhs2;

	/**
	 * Constructs a new binary rule.
	 * 
	 * @param lhs the left-hand side of the new rule
	 * @param rhs1 the first category on the right-hand side of the new rule
	 * @param rhs2 the second category on the right-hand side of the new rule
	 */
	public BinaryRule(int lhs, int rhs1, int rhs2) {
		super();
		this.lhs = lhs;
		this.rhs1 = rhs1;
		this.rhs2 = rhs2;
	}

	public int getLHS() {
		return lhs;
	}

	/**
	 * Returns the first category on the right-hand side of this rule.
	 * 
	 * @return the first category on the right-hand side of this rule
	 */
	public int getRHS1() {
		return rhs1;
	}

	/**
	 * Returns the second category on the right-hand side of this rule.
	 * 
	 * @return the second category on the right-hand side of this rule
	 */
	public int getRHS2() {
		return rhs2;
	}

	/**
	 * Tests whether this rule is equal to the specified object.
	 *
	 * @param obj the object to compare with
	 * @return <code>true</code> iff this rule is equal to the specified object
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final BinaryRule other = (BinaryRule) obj;
		if (this.lhs != other.lhs) {
			return false;
		}
		if (this.rhs1 != other.rhs1) {
			return false;
		}
		if (this.rhs2 != other.rhs2) {
			return false;
		}
		return true;
	}

	/**
	 * Returns the hash code of this rule.
	 *
	 * @return the hash code of this rule
	 */
	@Override
	public int hashCode() {
		int hash = 3;
		hash = 89 * hash + this.lhs;
		hash = 89 * hash + this.rhs1;
		hash = 89 * hash + this.rhs2;
		return hash;
	}
}
