/*
 * See the file "LICENSE" for the full license governing this code.
 */
package up;

import java.util.Deque;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;

/**
 * Evaluate the performance of a parser based on bracket scores.
 *
 * @author Marco Kuhlmann <marco.kuhlmann@lingfil.uu.se>
 */
public class BracketScorer {

    /**
     * Number of brackets in the gold trees.
     */
    private int nBracketsGold;
    /**
     * Number of brackets in the parsed trees.
     */
    private int nBracketsParsed;
    /**
     * Number of matched brackets.
     */
    private int nMatchedBrackets;

    /**
     * Constructs a new scorer.
     */
    public BracketScorer() {
    }

    /**
     * Updates the scores with the specified gold and parsed tree.
     *
     * @param gold the gold tree
     * @param parsed the parsed tree
     */
    public void update(Tree gold, Tree parsed) {
        Set<Bracket> bracketsGold = getBrackets(gold);
        nBracketsGold += bracketsGold.size();

        Set<Bracket> bracketsParsed = getBrackets(parsed);
        nBracketsParsed += bracketsParsed.size();

        // Intersect bracketsGold and bracketsParsed.
        bracketsGold.retainAll(bracketsParsed);
        nMatchedBrackets += bracketsGold.size();
    }

    /**
     * Returns the precision computed by this scorer.
     *
     * @return the precision computed by this scorer
     */
    public double getPrecision() {
        return nMatchedBrackets / (double) nBracketsParsed;
    }

    /**
     * Returns the recall computed by this scorer.
     *
     * @return the recall computed by this scorer
     */
    public double getRecall() {
        return nMatchedBrackets / (double) nBracketsGold;
    }

    /**
     * Returns the F-measure computed by this scorer.
     *
     * @return the F-measure computed by this scorer
     */
    public double getFMeasure() {
        double p = getPrecision();
        double r = getRecall();
        return 2.0 * p * r / (p + r);
    }

    /**
     * Returns the brackets for the specified tree.
     *
     * @param tree the tree whose brackets should be returned
     * @return the brackets for the specified tree
     */
    private static Set<Bracket> getBrackets(Tree tree) {
        BracketHandler bracketHandler = new BracketHandler();
        tree.traverseLeftToRight(bracketHandler);
        return bracketHandler.getBrackets();
    }

    /**
     * Handler used to collect the brackets from a tree.
     */
    private static class BracketHandler implements TraversalHandler {

        /**
         * The brackets collected so far.
         */
        private final Set<Bracket> brackets;
        /**
         * The minimal positions.
         */
        private final Deque<Integer> minPositions;
        /**
         * The current position (leaf index).
         */
        private int position;

        /**
         * Construct a new handler.
         */
        public BracketHandler() {
            this.brackets = new HashSet<Bracket>();
            this.minPositions = new LinkedList<Integer>();
        }

        /**
         * Returns the brackets collected so far.
         *
         * @return the set of brackets collected so far
         */
        public Set<Bracket> getBrackets() {
            return brackets;
        }

        public void enter(Tree tree) {
            // Remember that the current position is the left endpoint of the
            // span for the specified tree.
            minPositions.addLast(position);
        }

        public void leave(Tree tree) {
            // Retrieve the left endpoint of the span for the specified tree.
            int min = minPositions.removeLast();
            if (tree.isTerminal()) {
                // Do not collect a bracket for terminals; just update the position.
                position++;
            } else if (tree.getNSubtrees() == 1 && tree.getLeftmostSubtree().isTerminal()) {
				// Do not score preterminals.
			} else {
                // Collect a bracket.
                brackets.add(new Bracket(min, position, tree.getLabel()));
            }
        }
    }

    /**
     * A bracket.
     */
    private static class Bracket {

        /**
         * The left endpoint of the bracket.
         */
        public final int min;
        /**
         * The right endpoint of the bracket.
         */
        public final int max;
        /**
         * The label of the bracket.
         */
        public final String label;

        /**
         * Constructs a new bracket with the specified values.
         *
         * @param min the left endpoint of the bracket
         * @param max the right endpoint of the bracket
         * @param label the label of the bracket
         */
        public Bracket(int min, int max, String label) {
            this.min = min;
            this.max = max;
            this.label = label;
        }

        @Override
        public String toString() {
            return "[" + min + ", " + max + ", " + label + "]";
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final Bracket other = (Bracket) obj;
            if (this.min != other.min) {
                return false;
            }
            if (this.max != other.max) {
                return false;
            }
            if ((this.label == null) ? (other.label != null) : !this.label.equals(other.label)) {
                return false;
            }
            return true;
        }

        @Override
        public int hashCode() {
            int hash = 5;
            hash = 71 * hash + this.min;
            hash = 71 * hash + this.max;
            hash = 71 * hash + (this.label != null ? this.label.hashCode() : 0);
            return hash;
        }
    }
}
