/*
 * See the file "LICENSE" for the full license governing this code.
 */
package up;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

/**
 * A utility class that reads a grammar from a file.
 *
 * @author Marco Kuhlmann <marco.kuhlmann@lingfil.uu.se>
 */
public class GrammarReader {

	/**
	 * Private constructor to prevent instantiation.
	 */
	private GrammarReader() {
	}

	/**
	 * Reads a grammar from the specified file and returns it.
	 *
	 * @param fileName the name of the file that contains the grammar
	 * @return the grammar contained in the specified file
	 * @throws IOException if an error occurs duing the reading of the specified
	 * file
	 */
	public static Grammar read(String fileName) throws IOException {
		Grammar grammar = new Grammar();
		BufferedReader reader = new BufferedReader(new FileReader(fileName));
		int lineNumber = 0;
		try {
			while (true) {
				String line = reader.readLine();
				if (line == null) {
					break;
				}
				lineNumber++;
				String[] tokens = line.split("\\s");
				if (!(tokens.length == 4 || tokens.length == 5)) {
					throw new IOException("Cannot parse the rule in line no. " + lineNumber);
				} else {
					String lhs = tokens[0];
					Rule rule = null;

					if (tokens.length == 4) {
						if (!(tokens[1].equals("=>") || tokens[1].equals("->"))) {
							throw new IOException("Cannot parse the rule in line no. " + lineNumber);
						} else {
							String rhs = tokens[2];
							if (tokens[1].equals("=>")) {
								rule = grammar.addPreterminalRule(lhs, rhs);
							} else {
								rule = grammar.addUnaryRule(lhs, rhs);
							}
						}
					} else {
						if (!(tokens[1].equals("->"))) {
							throw new IOException("Cannot parse the rule in line no. " + lineNumber);
						} else {
							String rhs1 = tokens[2];
							String rhs2 = tokens[3];
							rule = grammar.addBinaryRule(lhs, rhs1, rhs2);
						}
					}

					try {
						double score = Double.parseDouble(tokens[tokens.length - 1]);
						rule.setScore(score);
					} catch (NumberFormatException e) {
						throw new IOException("Cannot parse the rule score in line no. " + lineNumber);
					}
				}
			}
		} finally {
			reader.close();
		}

		grammar.seal();

		System.err.format("Read a grammar with %d categories, %d words, and %d rules.%n", grammar.getNCategories(), grammar.getNWords(), grammar.getNRules());

		return grammar;
	}
}
