/*
 * See the file "LICENSE" for the full license governing this code.
 */
package up;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * An index for objects. An index maintains a bijective mapping between a set of
 * objects and a contiguous prefix of the non-negative integers. In the context
 * of this mapping, the objects are called <em>index entries</em> and the
 * integers are called <em>index values</em>.
 *
 * @param <E> the type of the index entries
 * @author Marco Kuhlmann <marco.kuhlmann@lingfil.uu.se>
 */
public class Index<E> {

	/**
	 * The entries of the index.
	 */
	private final List<E> entries;
	/**
	 * The index mapping.
	 */
	private final Map<E, Integer> map;

	/**
	 * Constructs an empty index.
	 */
	public Index() {
		this.entries = new ArrayList<E>();
		this.map = new HashMap<E, Integer>();
	}

	/**
	 * Adds the specified entry to this index.
	 *
	 * @param entry the entry to be added to this index
	 * @return the index value of the new entry
	 */
	public int add(E entry) {
		Integer i = map.get(entry);
		if (i == null) {
			i = entries.size();
			entries.add(entry);
			map.put(entry, i);
		}
		return i;
	}

	/**
	 * Return the canonical version of the specified entry in this index. This
	 * adds the entry to the index if it is not already present.
	 *
	 * @param entry the entry whose canonical version should be returned
	 * @return the canonical version of the specified entry in this index
	 */
	public E internalize(E entry) {
		Integer i = map.get(entry);
		if (i == null) {
			i = entries.size();
			entries.add(entry);
			map.put(entry, i);
			return entry;
		} else {
			return entries.get(i);
		}
	}

	/**
	 * Returns the entry with the specified index value.
	 *
	 * @param index the index value of the entry to be returned
	 * @return the entry with the specified index, or <code>null</code> if this
	 * index does not contain an entry with the specified index value
	 */
	public E getEntry(int index) {
		if (index < 0 || index >= entries.size()) {
			return null;
		} else {
			return entries.get(index);
		}
	}

	/**
	 * Returns the list of all entries of this index.
	 *
	 * @return the list of all entries of this index, in ascending order of
	 * their index values
	 */
	public List<E> getEntries() {
		return entries;
	}

	/**
	 * Returns the number of entries in this index.
	 *
	 * @return the number of entries in this index
	 */
	public int getNEntries() {
		return entries.size();
	}

	/**
	 * Returns the index value for the specified entry.
	 *
	 * @param entry the entry whose index value should be returned
	 * @return the index value for the specified entry, or <code>-1</code> if
	 * this index does not contain the specified entry
	 */
	public int getIndex(E entry) {
		Integer code = map.get(entry);
		if (code == null) {
			return -1;
		} else {
			return code;
		}
	}

	/**
	 * Tests whether this index contains the specified entry.
	 *
	 * @param entry the entry whose presence in this index is to be tested
	 * @return <code>true</code> if this index contains the specified entry
	 */
	public boolean contains(E entry) {
		return getIndex(entry) >= 0;
	}
}
