/*
 * See the file "LICENSE" for the full license governing this code.
 */
package up;

import java.io.*;
import java.util.*;

/**
 * Read files in the Penn Treebank format.
 *
 * @author Marco Kuhlmann <marco.kuhlmann@lingfil.uu.se>
 */
public class PennTreebankReader {

    /**
     * The label used for the root node of a tree.
     */
    public static final String ROOT_LABEL = "ROOT";
    /**
     * The tree handler used by this reader.
     */
    private TreeHandler handler;

    /**
     * Constructs a reader that sends the trees to the specified handler.
     *
     * @param handler the tree handler to send the read trees to
     */
    public PennTreebankReader(TreeHandler handler) {
        this.handler = handler;
    }

    private void readInternal(File file) throws FileNotFoundException, IOException {
        BufferedReader reader = new BufferedReader(new FileReader(file));
        Deque<Tree> trees = new LinkedList<Tree>();
        String line;
        boolean readLabel = false;
        while ((line = reader.readLine()) != null) {
            int indexL = 0;
            int indexR = 0;
            while (indexR < line.length()) {
                char c = line.charAt(indexR);
                if (Character.isWhitespace(c)) {
                    indexR++;
                    indexL = indexR;
                    continue;
                }
                if (c == '(') {
                    indexR++;
                    trees.addFirst(new Tree(ROOT_LABEL));
                    readLabel = true;
                    indexL = indexR;
                    continue;
                }
                if (c == ')') {
                    indexR++;
                    Tree tree = trees.removeFirst();
                    if (trees.isEmpty()) {
                        handler.next(tree);
                    } else {
                        trees.getFirst().addAsRightmostSubtree(tree);
                    }
                    indexL = indexR;
                    continue;
                }
                while (indexR < line.length() && !(Character.isWhitespace(c) || c == '(' || c == ')')) {
                    indexR++;
                    c = line.charAt(indexR);
                }
                String text = line.substring(indexL, indexR);
                if (readLabel) {
                    assert text.length() > 0;
                    trees.getFirst().setLabel(text);
                    readLabel = false;
                } else {
                    trees.getFirst().addAsRightmostSubtree(new Tree(text));
                }
                indexL = indexR;
            }
        }
        reader.close();
    }

    /**
     * Reads trees from the specified file.
     *
     * @param file the file to read
     * @throws FileNotFoundException if the specified file cannot be found
     * @throws IOException if an error occurs during the reading of the
     * specified file
     */
    public void read(File file) throws FileNotFoundException, IOException {
        handler.init();
        readInternal(file);
        handler.exit();
    }

    /**
     * Reads trees from the specified range of WSJ data.
     *
     * @param root the root of the file structure containing the WSJ data
     * @param from the lowest file number to be read from the specified file
     * structure
     * @param to the highest file number to be read from the specified file
     * structure
     * @throws FileNotFoundException if a file cannot be found
     * @throws IOException if an error occurs during reading
     */
    public void read(File root, int from, int to) throws FileNotFoundException, IOException {
        FileFilter filter = new MyFileFilter(from, to);
        List<File> files = new ArrayList<File>();

        Deque<File> agenda = new LinkedList<File>();
        agenda.addLast(root);
        while (!agenda.isEmpty()) {
            File next = agenda.removeLast();
            if (filter.accept(next)) {
                files.add(next);
                continue;
            }
            if (next.isDirectory()) {
                for (File child : next.listFiles()) {
                    agenda.addLast(child);
                }
                continue;
            }
        }
        Collections.sort(files);

        handler.init();
        for (File file : files) {
            readInternal(file);
        }
        handler.exit();
    }

    private static class MyFileFilter implements FileFilter {

        private final int lo;
        private final int hi;

        public MyFileFilter(int lo, int hi) {
            this.lo = lo;
            this.hi = hi;
        }

        public boolean accept(File file) {
            // Accepts with a name of the form wsj_DDDD.mrg.
            if (!file.isFile()) {
                return false;
            } else {
                String name = file.getName();
                if (!(name.startsWith("wsj_") && name.endsWith(".mrg"))) {
                    return false;
                } else {
                    try {
                        int number = Integer.parseInt(name.substring(4, 8));
                        return lo <= number && number <= hi;
                    } catch (NumberFormatException e) {
                        return false;
                    }
                }
            }
        }
    }
}
